/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach.muk;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.web.client.RestClientException;

import de.ozgcloud.muk.elster.transfer.AuftragAngelegt;
import de.ozgcloud.nachrichten.postfach.muk.transfer.ElsterTransferRemoteService;

class MukPostfachRemoteServiceTest {
	@Spy
	@InjectMocks
	private MukPostfachRemoteService mukPostfachRemoteService;

	@Mock
	private ElsterTransferRemoteService elsterTransferService;

	@Nested
	class TestSendingNachricht {
		@BeforeEach
		void setUp() {
			when(elsterTransferService.send(any())).thenReturn(new AuftragAngelegt());
		}

		@Test
		void shouldSendMitteilung() {
			mukPostfachRemoteService.sendMessage(PostfachNachrichtTestFactory.createBuilder().attachments(null).build());

			verify(elsterTransferService).send(any());
		}

		@Test
		void shouldSendBescheid() {
			mukPostfachRemoteService.sendMessage(PostfachNachrichtTestFactory.create());

			verify(elsterTransferService).send(any());
		}

		@Test
		void shouldThrowException() {
			when(elsterTransferService.send(any())).thenThrow(new RestClientException("Test error"));
			var message = PostfachNachrichtTestFactory.create();

			assertThatExceptionOfType(MukServerException.class).isThrownBy(() -> mukPostfachRemoteService.sendMessage(message));
		}
	}

	@Nested
	class TestUnimplemented {
		@Test
		void unimplementedDeleteMessage() {
			assertThatNoException().isThrownBy(() -> mukPostfachRemoteService.deleteMessage(""));
		}
	}

	@Nested
	class TestGetPostfachType {
		@Test
		void shouldGetTypeMUK() {
			var type = mukPostfachRemoteService.getPostfachType();

			assertThat(type).isEqualTo(MukPostfachRemoteService.POSTFACH_TYPE_MUK);
		}
	}

	@Nested
	class TestReplyAllowed {
		@Test
		void shouldAllowReply() {
			var replyAllowed = mukPostfachRemoteService.isReplyAllowed();

			assertThat(replyAllowed).isTrue();
		}
	}
}