/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.nachrichten.postfach.muk;

import static de.ozgcloud.nachrichten.postfach.PostfachNachricht.ReplyOption.*;

import java.time.ZonedDateTime;
import java.util.List;
import java.util.UUID;

import org.bson.types.ObjectId;

import com.thedeanda.lorem.LoremIpsum;

import de.ozgcloud.nachrichten.postfach.PostfachAddress;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht.Direction;

public class PostfachNachrichtTestFactory {

	public static final String ID = ObjectId.get().toHexString();
	public static final String CREATED_AT_STR = "2020-04-01T10:30:10Z";
	public static final ZonedDateTime CREATED_AT = ZonedDateTime.parse(CREATED_AT_STR);
	public static final String CREATED_BY = UUID.randomUUID().toString();
	public static final String MAIL_BODY = "Body\nString";

	public static final String SENT_AT_STR = "2020-04-01T11:30:10Z";
	public static final ZonedDateTime SENT_AT = ZonedDateTime.parse(SENT_AT_STR);
	public static final boolean SENT_SUCCESSFUL = true;
	public static final PostfachNachricht.ReplyOption REPLY_OPTION = MANDATORY;

	public static final Direction DIRECTION = Direction.IN;

	public static final String ATTACHMENT_FILE_ID = "21";
	public static final List<String> ATTACHMENTS = List.of(ATTACHMENT_FILE_ID);
	public static final String POSTFACH_ID = "ek-84fe3bbb0aabc4449bb99bfffdc2533691320ad3";
	public static final String POSTFACH_SERVICE_KONTO_TYPE = "MUK";
	public static final int POSTFACH_TYPE = 1;
	public static final String SUBJECT = LoremIpsum.getInstance().getTitle(1);
	public static final String MESSAGE_ID = UUID.randomUUID().toString();
	public static final String NACHRICHT_ID = ObjectId.get().toHexString();

	public static PostfachNachricht create() {
		return createBuilder().build();
	}

	public static PostfachNachricht.PostfachNachrichtBuilder createBuilder() {
		return PostfachNachricht.builder()
		  .id(ID)
		  .postfachAddress(postfachAddress())
		  .messageId(MESSAGE_ID)
		  .vorgangId(NACHRICHT_ID)
		  .direction(DIRECTION)
		  .createdAt(CREATED_AT)
		  .createdBy(CREATED_BY)
		  .sentAt(SENT_AT)
		  .sentSuccessful(SENT_SUCCESSFUL)
		  .replyOption(REPLY_OPTION)
		  .subject(SUBJECT)
		  .mailBody(MAIL_BODY)
		  .attachments(ATTACHMENTS);
	}

	public static PostfachAddress postfachAddress() {
		return PostfachAddress.builder()
		  .type(POSTFACH_TYPE)
		  .serviceKontoType(POSTFACH_SERVICE_KONTO_TYPE)
		  .identifier(MukPostfachAddressIdentifier.builder().postfachId(POSTFACH_ID).build()).build();
	}
}
